# functions for scoring gRNA sequences

def aziWS(s,c,p):
    '''Use Microsoft Azimuth web service for CRISPR sgRNA efficacy
    prediction to obtain on-target scores for given gRNA sequences; return
    list of such scores; s=list of 30-base sequences, each equal to a
    candidate gRNA sequence preceded by 4 upstream bases and followed
    by 6 downstream bases; c=list of "Target Cut Length" values;
    p=list of "Target Cut Percent" values; '''
    v=[[i[0]]+[str(int(round(i))) for i in i[1:]] for i in zip(s,c,p)]
    d={"Inputs":{"input1":{
        "ColumnNames":["sequence","cutsite","percentpeptide"],
        "Values":v}},"GlobalParameters":{}}
    import urllib2 as ul,json
    b=str.encode(json.dumps(d)) # body of request
    u="https://ussouthcentral.services.azureml.net/workspaces/"+\
      "ee5485c1d9814b8d8c647a89db12d4df/services/c24d128abfaf4832abf1e7ef"+\
      "45db4b54/execute?api-version=2.0&details=true" # URL
    k="api-key-goes-here" # API key for the web service
    h={"Content-Type":"application/json","Authorization":("Bearer "+k)} #header
    try: r=ul.urlopen(ul.Request(u,b,h)).read()
    except ul.HTTPError as x:
        s="HTTP request failed with code: "+str(x.code)
        s+='\n'+str(x.info()) # print headers
        s+='\n'+json.loads(x.read())
        raise Exception(s)
    r=json.loads(r)
    r=[float(i[0]) for i in r["Results"]["output2"]["value"]["Values"]]
    return r

def aziMC(s,c,p):
    '''Use module azimuth.model_comparison to obtain on-target scores
    for specified gRNA sequences; s,c,p=same as for function aziWS;'''
    import azimuth.model_comparison as mc,numpy as np
    s,c,p=(np.asarray(i) for i in (s,c,p))
    r=mc.predict(s,c,p)
    return r

if __name__=="__main__":
    s=["AGAAATTGACCTTCGCAACCCCAAGGGGAA","GTTCGTAACGGGAATGCCGAAGCGTGGGAA",
    "TTGTGCTGTGTGGAGACCTCAATGTGGCAC","GATGAATGCTCGATCCAAGAATGTTGGTTG"]
    c=[669,9,635,831]
    p=[69.9,0.9,66.4,86.8]
    c=p=[-1]*len(s)
    #c=p=[0]*len(s)
    r=aziWS(s,c,p)
    print("aziWS: "+str(r))
    r=aziMC(s,c,p)
    print("aziMC: "+str(r))
